@@ -21,7 +21,7 @@ from .core import (angular as _angular,
                    vector as _vector)
 from .core.hints import QuaternaryPointFunction as _QuaternaryPointFunction
 
-_QuaternaryFunction = _QuaternaryPointFunction[_hints.Coordinate]
+_QuaternaryFunction = _QuaternaryPointFunction[_hints.Scalar]
 Kind = _enums.Kind
 Orientation = _enums.Orientation
 Relation = _enums.Relation
@@ -37,15 +37,18 @@ class Mode(_enum.IntEnum):
 
 class Context:
     """Represents common language for computational geometry."""
-    __slots__ = ('_box_cls', '_centroidal', '_contour_cls', '_incircle',
-                 '_linear', '_measured', '_metric', '_mode', '_multipoint_cls',
-                 '_multipolygon_cls', '_multisegment_cls', '_point_cls',
-                 '_polygon_cls', '_segment_cls', '_sqrt', '_vector')
+    __slots__ = ('_box_cls', '_centroidal', '_contour_cls', '_empty',
+                 '_empty_cls', '_incircle', '_linear', '_measured', '_metric',
+                 '_mix_cls', '_mode', '_multipoint_cls', '_multipolygon_cls',
+                 '_multisegment_cls', '_point_cls', '_polygon_cls',
+                 '_segment_cls', '_sqrt', '_vector')
 
     def __init__(self,
                  *,
                  box_cls: _Type[_hints.Box] = _geometries.Box,
                  contour_cls: _Type[_hints.Contour] = _geometries.Contour,
+                 empty_cls: _Type[_hints.Empty] = _geometries.Empty,
+                 mix_cls: _Type[_hints.Mix] = _geometries.Mix,
                  multipoint_cls: _Type[_hints.Multipoint]
                  = _geometries.Multipoint,
                  multipolygon_cls: _Type[_hints.Multipolygon]
@@ -56,10 +59,12 @@ class Context:
                  polygon_cls: _Type[_hints.Polygon] = _geometries.Polygon,
                  segment_cls: _Type[_hints.Segment] = _geometries.Segment,
                  mode: Mode = Mode.EXACT,
-                 sqrt: _Callable[[_hints.Coordinate], _hints.Coordinate]
+                 sqrt: _Callable[[_hints.Scalar], _hints.Scalar]
                  = _sqrt) -> None:
         self._box_cls = box_cls
         self._contour_cls = contour_cls
+        self._empty, self._empty_cls = empty_cls(), empty_cls
+        self._mix_cls = mix_cls
         self._multipoint_cls = multipoint_cls
         self._multipolygon_cls = multipolygon_cls
         self._multisegment_cls = multisegment_cls
@@ -166,6 +171,21 @@ class Context:
         """
         return self._vector.dot_product
 
+    @property
+    def empty(self) -> _hints.Empty:
+        """Returns an empty geometry."""
+        return self._empty
+
+    @property
+    def empty_cls(self) -> _Type[_hints.Empty]:
+        """Returns type for empty geometries."""
+        return self._empty_cls
+
+    @property
+    def mix_cls(self) -> _Type[_hints.Mix]:
+        """Returns type for mixes."""
+        return self._mix_cls
+
     @property
     def mode(self) -> Mode:
         """Returns mode of the context."""
@@ -247,20 +267,23 @@ class Context:
     @property
     def region_signed_area(self) -> _measured.SignedRegionMeasure:
         """
-        Returns signed area of the region given its contour vertices.
+        Returns signed area of the region given its contour.
 
         Time complexity:
-            ``O(len(vertices))``
+            ``O(len(contour.vertices))``
         Memory complexity:
             ``O(1)``
 
         >>> context = get_context()
+        >>> Contour = context.contour_cls
         >>> Point = context.point_cls
-        >>> context.region_signed_area([Point(0, 0), Point(1, 0), Point(1, 1),
-        ...                             Point(0, 1)]) == 1
+        >>> (context.region_signed_area(Contour([Point(0, 0), Point(1, 0),
+        ...                                      Point(1, 1), Point(0, 1)]))
+        ...  == 1)
         True
-        >>> context.region_signed_area([Point(0, 0), Point(0, 1), Point(1, 1),
-        ...                             Point(1, 0)]) == -1
+        >>> (context.region_signed_area(Contour([Point(0, 0), Point(0, 1),
+        ...                                      Point(1, 1), Point(1, 0)]))
+        ...  == -1)
         True
         """
         return self._measured.signed_region_measure
@@ -271,7 +294,7 @@ class Context:
         return self._segment_cls
 
     @property
-    def sqrt(self) -> _Callable[[_hints.Coordinate], _hints.Coordinate]:
+    def sqrt(self) -> _Callable[[_hints.Scalar], _hints.Scalar]:
         """Returns function for computing square root."""
         return self._sqrt
 
@@ -331,11 +354,10 @@ class Context:
 
     def box_segment_squared_distance(self,
                                      box: _hints.Box,
-                                     start: _hints.Point,
-                                     end: _hints.Point) -> _hints.Coordinate:
+                                     segment: _hints.Segment
+                                     ) -> _hints.Scalar:
         """
-        Returns squared Euclidean distance between box and a segment
-        given its endpoints.
+        Returns squared Euclidean distance between box and a segment.
 
         Time complexity:
             ``O(1)``
@@ -343,19 +365,21 @@ class Context:
             ``O(1)``
 
         >>> context = get_context()
-        >>> Box, Point = context.box_cls, context.point_cls
-        >>> context.box_segment_squared_distance(Box(0, 1, 0, 1), Point(0, 0),
-        ...                                      Point(1, 1)) == 0
+        >>> Box = context.box_cls
+        >>> Point = context.point_cls
+        >>> Segment = context.segment_cls
+        >>> context.box_segment_squared_distance(
+        ...     Box(0, 1, 0, 1), Segment(Point(0, 0), Point(1, 1))) == 0
         True
-        >>> context.box_segment_squared_distance(Box(0, 1, 0, 1), Point(2, 0),
-        ...                                      Point(2, 1)) == 1
+        >>> context.box_segment_squared_distance(
+        ...     Box(0, 1, 0, 1), Segment(Point(2, 0), Point(2, 1))) == 1
         True
-        >>> context.box_segment_squared_distance(Box(0, 1, 0, 1), Point(2, 2),
-        ...                                      Point(3, 2)) == 2
+        >>> context.box_segment_squared_distance(
+        ...     Box(0, 1, 0, 1), Segment(Point(2, 2), Point(3, 2))) == 2
         True
         """
         return self._metric.box_segment_squared_metric(
-                box, start, end, self.dot_product, self.segments_relation,
+                box, segment, self.dot_product, self._segments_intersect,
                 self.point_cls)
 
     def contour_box(self, contour: _hints.Contour) -> _hints.Box:
@@ -379,25 +403,51 @@ class Context:
         """
         return _boxed.from_contour(contour, self.box_cls)
 
-    def contour_centroid(self, vertices: _Sequence[_hints.Point]
-                         ) -> _hints.Point:
+    def contour_centroid(self, contour: _hints.Contour) -> _hints.Point:
         """
-        Constructs centroid of a contour given its vertices.
+        Constructs centroid of a contour.
 
         Time complexity:
-            ``O(len(vertices))``
+            ``O(len(contour.vertices))``
         Memory complexity:
             ``O(1)``
 
         >>> context = get_context()
-        >>> Point = context.point_cls
-        >>> context.contour_centroid([Point(0, 0), Point(2, 0), Point(2, 2),
-        ...                           Point(0, 2)]) == Point(1, 1)
+        >>> Contour, Point = context.contour_cls, context.point_cls
+        >>> (context.contour_centroid(Contour([Point(0, 0), Point(2, 0),
+        ...                                    Point(2, 2), Point(0, 2)]))
+        ...  == Point(1, 1))
         True
         """
-        return self._centroidal.contour_centroid(vertices, self.point_cls,
+        return self._centroidal.contour_centroid(contour, self.point_cls,
                                                  self.sqrt)
 
+    def contour_edges(self, contour: _hints.Contour
+                      ) -> _Sequence[_hints.Segment]:
+        """
+        Constructs edges of a contour.
+
+        Time complexity:
+            ``O(len(contour.vertices))``
+        Memory complexity:
+            ``O(1)``
+
+        >>> context = get_context()
+        >>> Contour = context.contour_cls
+        >>> Point = context.point_cls
+        >>> Segment = context.segment_cls
+        >>> (context.contour_edges(Contour([Point(0, 0), Point(2, 0),
+        ...                                 Point(2, 2), Point(0, 2)]))
+        ...  == [Segment(Point(0, 2), Point(0, 0)),
+        ...      Segment(Point(0, 0), Point(2, 0)),
+        ...      Segment(Point(2, 0), Point(2, 2)),
+        ...      Segment(Point(2, 2), Point(0, 2))])
+        True
+        """
+        segment_cls, vertices = self.segment_cls, contour.vertices
+        return [segment_cls(vertices[index - 1], vertices[index])
+                for index in range(len(vertices))]
+
     def contours_box(self, contours: _Sequence[_hints.Contour]) -> _hints.Box:
         """
         Constructs box from contours.
@@ -411,8 +461,9 @@ class Context:
  for contour in contours)``.
 
         >>> context = get_context()
-        >>> Box, Contour, Point = (context.box_cls, context.contour_cls,
-        ...                        context.point_cls)
+        >>> Box = context.box_cls
+        >>> Contour = context.contour_cls
+        >>> Point = context.point_cls
         >>> (context.contours_box([Contour([Point(0, 0), Point(1, 0),
         ...                                 Point(1, 1), Point(0, 1)]),
         ...                        Contour([Point(1, 1), Point(2, 1),
@@ -422,6 +473,36 @@ class Context:
         """
         return _boxed.from_contours(contours, self.box_cls)
 
+    def is_region_convex(self, contour: _hints.Contour) -> bool:
+        """
+        Checks if region (given its contour) is convex.
+
+        Time complexity:
+            ``O(len(contour.vertices))``
+        Memory complexity:
+            ``O(1)``
+
+        >>> context = get_context()
+        >>> Contour = context.contour_cls
+        >>> Point = context.point_cls
+        >>> context.is_region_convex(Contour([Point(0, 0), Point(3, 0),
+        ...                                   Point(1, 1), Point(0, 3)]))
+        False
+        >>> context.is_region_convex(Contour([Point(0, 0), Point(2, 0),
+        ...                                   Point(2, 2), Point(0, 2)]))
+        True
+        """
+        vertices = contour.vertices
+        vertices_count = len(vertices)
+        if vertices_count == 3:
+            return True
+        orienteer = self.angle_orientation
+        base_orientation = orienteer(vertices[-2], vertices[-1], vertices[0])
+        # orientation change means that internal angle is greater than 180°
+        return all(orienteer(vertices[index - 1], vertices[index],
+                             vertices[index + 1]) is base_orientation
+                   for index in range(vertices_count - 1))
+
     def merged_box(self, first_box: _hints.Box, second_box: _hints.Box
                    ) -> _hints.Box:
         """
@@ -438,30 +519,35 @@ class Context:
         ...  == Box(0, 2, 0, 2))
         True
         """
-        return _boxed.merge(first_box, second_box, self.box_cls)
+        return self.box_cls(min(first_box.min_x, second_box.min_x),
+                            max(first_box.max_x, second_box.max_x),
+                            min(first_box.min_y, second_box.min_y),
+                            max(first_box.max_y, second_box.max_y))
 
-    def multipoint_centroid(self, points: _Sequence[_hints.Point]
+    def multipoint_centroid(self, multipoint: _hints.Multipoint
                             ) -> _hints.Point:
         """
-        Constructs centroid of a multipoint given its points.
+        Constructs centroid of a multipoint.
 
         Time complexity:
-            ``O(len(points))``
+            ``O(len(multipoint.points))``
         Memory complexity:
             ``O(1)``
 
         >>> context = get_context()
+        >>> Multipoint = context.multipoint_cls
         >>> Point = context.point_cls
-        >>> context.multipoint_centroid([Point(0, 0), Point(2, 0), Point(2, 2),
-        ...                              Point(0, 2)]) == Point(1, 1)
+        >>> context.multipoint_centroid(
+        ...     Multipoint([Point(0, 0), Point(2, 0), Point(2, 2),
+        ...                 Point(0, 2)])) == Point(1, 1)
         True
         """
-        return self._centroidal.multipoint_centroid(points, self.point_cls)
+        return self._centroidal.multipoint_centroid(multipoint, self.point_cls)
 
-    def multipolygon_centroid(self, polygons: _Sequence[_hints.Polygon]
+    def multipolygon_centroid(self, multipolygon: _hints.Multipolygon
                               ) -> _hints.Point:
         """
-        Constructs centroid of a multipolygon given its polygons.
+        Constructs centroid of a multipolygon.
 
         Time complexity:
             ``O(len(vertices_count))``
@@ -470,47 +556,51 @@ class Context:
 
         where ``vertices_count = sum(len(polygon.border.vertices)\
  + sum(len(hole.vertices) for hole in polygon.holes)\
- for polygon in polygons)``.
+ for polygon in multipolygon.polygons)``.
 
         >>> context = get_context()
-        >>> Contour, Point, Polygon = (context.contour_cls, context.point_cls,
-        ...                            context.polygon_cls)
+        >>> Contour = context.contour_cls
+        >>> Point = context.point_cls
+        >>> Polygon = context.polygon_cls
+        >>> Multipolygon = context.multipolygon_cls
         >>> (context.multipolygon_centroid(
-        ...     [Polygon(Contour([Point(0, 0), Point(1, 0), Point(1, 1),
-        ...                       Point(0, 1)]), []),
-        ...      Polygon(Contour([Point(1, 1), Point(2, 1), Point(2, 2),
-        ...                       Point(1, 2)]), [])])
+        ...      Multipolygon([Polygon(Contour([Point(0, 0), Point(1, 0),
+        ...                                     Point(1, 1), Point(0, 1)]),
+        ...                            []),
+        ...                    Polygon(Contour([Point(1, 1), Point(2, 1),
+        ...                                     Point(2, 2), Point(1, 2)]),
+        ...                            [])]))
         ...  == Point(1, 1))
         True
         """
-        return self._centroidal.multipolygon_centroid(polygons, self.point_cls)
+        return self._centroidal.multipolygon_centroid(multipolygon,
+                                                      self.point_cls)
 
-    def multisegment_centroid(self, segments: _Sequence[_hints.Segment]
+    def multisegment_centroid(self, multisegment: _hints.Multisegment
                               ) -> _hints.Point:
         """
-        Constructs centroid of a multisegment given its segments.
+        Constructs centroid of a multisegment.
 
         Time complexity:
-            ``O(len(vertices_count))``
+            ``O(len(multisegment.segments))``
         Memory complexity:
             ``O(1)``
 
-        where ``vertices_count = sum(len(segment.border.vertices)\
- + sum(len(hole.vertices) for hole in segment.holes)\
- for segment in segments)``.
-
         >>> context = get_context()
-        >>> Contour, Point, Segment = (context.contour_cls, context.point_cls,
-        ...                            context.segment_cls)
-        >>> (context.multisegment_centroid([Segment(Point(0, 0), Point(2, 0)),
-        ...                                 Segment(Point(2, 0), Point(2, 2)),
-        ...                                 Segment(Point(0, 2), Point(2, 2)),
-        ...                                 Segment(Point(0, 0), Point(0, 2))])
+        >>> Contour = context.contour_cls
+        >>> Point = context.point_cls
+        >>> Segment = context.segment_cls
+        >>> Multisegment = context.multisegment_cls
+        >>> (context.multisegment_centroid(
+        ...      Multisegment([Segment(Point(0, 0), Point(2, 0)),
+        ...                    Segment(Point(2, 0), Point(2, 2)),
+        ...                    Segment(Point(0, 2), Point(2, 2)),
+        ...                    Segment(Point(0, 0), Point(0, 2))]))
         ...  == Point(1, 1))
         True
         """
-        return self._centroidal.multisegment_centroid(segments, self.point_cls,
-                                                      self.sqrt)
+        return self._centroidal.multisegment_centroid(
+                multisegment, self.point_cls, self.sqrt)
 
     def points_convex_hull(self,
                            points: _Sequence[_hints.Point]
@@ -574,30 +664,30 @@ class Context:
         """
         return _boxed.from_polygon(polygon, self.box_cls)
 
-    def polygon_centroid(self,
-                         border: _hints.Contour,
-                         holes: _Sequence[_hints.Contour]) -> _hints.Point:
+    def polygon_centroid(self, polygon: _hints.Polygon) -> _hints.Point:
         """
-        Constructs centroid of a polygon given its border & holes.
+        Constructs centroid of a polygon.
 
         Time complexity:
             ``O(vertices_count)``
         Memory complexity:
             ``O(1)``
 
-        where ``vertices_count = len(border.vertices)\
- + sum(len(hole.vertices) for hole in holes)``.
+        where ``vertices_count = len(polygon.border.vertices)\
+ + sum(len(hole.vertices) for hole in polygon.holes)``.
 
         >>> context = get_context()
-        >>> Contour, Point = context.contour_cls, context.point_cls
-        >>> (context.polygon_centroid(Contour([Point(0, 0), Point(4, 0),
-        ...                                    Point(4, 4), Point(0, 4)]),
-        ...                           [Contour([Point(1, 1), Point(1, 3),
-        ...                                     Point(3, 3), Point(3, 1)])])
-        ...  == Point(2, 2))
+        >>> Contour = context.contour_cls
+        >>> Point = context.point_cls
+        >>> Polygon = context.polygon_cls
+        >>> context.polygon_centroid(
+        ...     Polygon(Contour([Point(0, 0), Point(4, 0), Point(4, 4),
+        ...                      Point(0, 4)]),
+        ...             [Contour([Point(1, 1), Point(1, 3), Point(3, 3),
+        ...                       Point(3, 1)])])) == Point(2, 2)
         True
         """
-        return self._centroidal.polygon_centroid(border, holes, self.point_cls)
+        return self._centroidal.polygon_centroid(polygon, self.point_cls)
 
     def polygons_box(self, polygons: _Sequence[_hints.Polygon]) -> _hints.Box:
         """
@@ -625,23 +715,24 @@ class Context:
         """
         return _boxed.from_polygons(polygons, self.box_cls)
 
-    def region_centroid(self, vertices: _Sequence[_hints.Point]
-                        ) -> _hints.Point:
+    def region_centroid(self, contour: _hints.Contour) -> _hints.Point:
         """
-        Constructs centroid of a region given its contour vertices.
+        Constructs centroid of a region given its contour.
 
         Time complexity:
-            ``O(len(vertices))``
+            ``O(len(contour.vertices))``
         Memory complexity:
             ``O(1)``
 
         >>> context = get_context()
+        >>> Contour = context.contour_cls
         >>> Point = context.point_cls
-        >>> context.region_centroid([Point(0, 0), Point(2, 0), Point(2, 2),
-        ...                          Point(0, 2)]) == Point(1, 1)
+        >>> (context.region_centroid(Contour([Point(0, 0), Point(2, 0),
+        ...                                   Point(2, 2), Point(0, 2)]))
+        ...  == Point(1, 1))
         True
         """
-        return self._centroidal.region_centroid(vertices, self.point_cls)
+        return self._centroidal.region_centroid(contour, self.point_cls)
 
     def segment_box(self, segment: _hints.Segment) -> _hints.Box:
         """
@@ -655,18 +746,34 @@ class Context:
         >>> context = get_context()
         >>> Box, Point, Segment = (context.box_cls, context.point_cls,
         ...                        context.segment_cls)
-        >>> (context.segment_box(Segment(Point(0, 0), Point(1, 1)))
-        ...  == Box(0, 1, 0, 1))
+        >>> (context.segment_box(Segment(Point(0, 1), Point(2, 3)))
+        ...  == Box(0, 2, 1, 3))
         True
         """
         return _boxed.from_segment(segment, self.box_cls)
 
+    def segment_centroid(self, segment: _hints.Segment) -> _hints.Point:
+        """
+        Constructs centroid of a segment.
+
+        Time complexity:
+            ``O(1)``
+        Memory complexity:
+            ``O(1)``
+
+        >>> context = get_context()
+        >>> Point, Segment = context.point_cls, context.segment_cls
+        >>> (context.segment_centroid(Segment(Point(0, 1), Point(2, 3)))
+        ...  == Point(1, 2))
+        True
+        """
+        return self._centroidal.segment_centroid(segment, self.point_cls)
+
     def segment_contains_point(self,
-                               start: _hints.Point,
-                               end: _hints.Point,
+                               segment: _hints.Segment,
                                point: _hints.Point) -> bool:
         """
-        Checks if a segment given by its endpoints contains given point.
+        Checks if a segment contains given point.
 
         Time complexity:
             ``O(1)``
@@ -675,36 +782,35 @@ class Context:
 
         >>> context = get_context()
         >>> Point = context.point_cls
-        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        >>> Segment = context.segment_cls
+        >>> context.segment_contains_point(Segment(Point(0, 0), Point(2, 0)),
         ...                                Point(0, 0))
         True
-        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        >>> context.segment_contains_point(Segment(Point(0, 0), Point(2, 0)),
         ...                                Point(0, 2))
         False
-        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        >>> context.segment_contains_point(Segment(Point(0, 0), Point(2, 0)),
         ...                                Point(1, 0))
         True
-        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        >>> context.segment_contains_point(Segment(Point(0, 0), Point(2, 0)),
         ...                                Point(1, 1))
         False
-        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        >>> context.segment_contains_point(Segment(Point(0, 0), Point(2, 0)),
         ...                                Point(2, 0))
         True
-        >>> context.segment_contains_point(Point(0, 0), Point(2, 0),
+        >>> context.segment_contains_point(Segment(Point(0, 0), Point(2, 0)),
         ...                                Point(3, 0))
         False
         """
-        return self._linear.containment_checker(start, end, point,
-                                                self.cross_product)
+        return self._linear.containment_checker(segment.start, segment.end,
+                                                point, self.cross_product)
 
     def segment_point_squared_distance(self,
-                                       start: _hints.Point,
-                                       end: _hints.Point,
+                                       segment: _hints.Segment,
                                        point: _hints.Point
-                                       ) -> _hints.Coordinate:
+                                       ) -> _hints.Scalar:
         """
-        Returns squared Euclidean distance between segment given its endpoints
-        and a point.
+        Returns squared Euclidean distance between segment and a point.
 
         Time complexity:
             ``O(1)``
@@ -713,18 +819,19 @@ class Context:
 
         >>> context = get_context()
         >>> Point = context.point_cls
-        >>> context.segment_point_squared_distance(Point(0, 0), Point(1, 0),
-        ...                                        Point(0, 0)) == 0
+        >>> Segment = context.segment_cls
+        >>> context.segment_point_squared_distance(
+        ...     Segment(Point(0, 0), Point(1, 0)), Point(0, 0)) == 0
         True
-        >>> context.segment_point_squared_distance(Point(0, 0), Point(1, 0),
-        ...                                        Point(0, 1)) == 1
+        >>> context.segment_point_squared_distance(
+        ...     Segment(Point(0, 0), Point(1, 0)), Point(0, 1)) == 1
         True
-        >>> context.segment_point_squared_distance(Point(0, 0), Point(1, 0),
-        ...                                        Point(2, 1)) == 2
+        >>> context.segment_point_squared_distance(
+        ...     Segment(Point(0, 0), Point(1, 0)), Point(2, 1)) == 2
         True
         """
-        return self._metric.segment_point_squared_metric(start, end, point,
-                                                         self.dot_product)
+        return self._metric.segment_point_squared_metric(
+                segment.start, segment.end, point, self.dot_product)
 
     def segments_box(self, segments: _Sequence[_hints.Segment]) -> _hints.Box:
         """
@@ -746,10 +853,8 @@ class Context:
         return _boxed.from_segments(segments, self.box_cls)
 
     def segments_intersection(self,
-                              first_start: _hints.Point,
-                              first_end: _hints.Point,
-                              second_start: _hints.Point,
-                              second_end: _hints.Point) -> _hints.Point:
+                              first: _hints.Segment,
+                              second: _hints.Segment) -> _hints.Point:
         """
         Returns intersection point of two segments.
 
@@ -760,28 +865,27 @@ class Context:
 
         >>> context = get_context()
         >>> Point = context.point_cls
-        >>> (context.segments_intersection(Point(0, 0), Point(2, 0),
-        ...                                Point(0, 0), Point(0, 1))
+        >>> Segment = context.segment_cls
+        >>> (context.segments_intersection(Segment(Point(0, 0), Point(2, 0)),
+        ...                                Segment(Point(0, 0), Point(0, 1)))
         ...  == Point(0, 0))
         True
-        >>> (context.segments_intersection(Point(0, 0), Point(2, 0),
-        ...                                Point(1, 0), Point(1, 1))
+        >>> (context.segments_intersection(Segment(Point(0, 0), Point(2, 0)),
+        ...                                Segment(Point(1, 0), Point(1, 1)))
         ...  == Point(1, 0))
         True
-        >>> (context.segments_intersection(Point(0, 0), Point(2, 0),
-        ...                                Point(2, 0), Point(3, 0))
+        >>> (context.segments_intersection(Segment(Point(0, 0), Point(2, 0)),
+        ...                                Segment(Point(2, 0), Point(3, 0)))
         ...  == Point(2, 0))
         True
         """
-        return self._linear.intersector(
-                first_start, first_end, second_start, second_end,
-                self.cross_product, self.point_cls)
+        return self._linear.intersector(first.start, first.end, second.start,
+                                        second.end, self.cross_product,
+                                        self.point_cls)
 
     def segments_relation(self,
-                          test_start: _hints.Point,
-                          test_end: _hints.Point,
-                          goal_start: _hints.Point,
-                          goal_end: _hints.Point) -> Relation:
+                          test: _hints.Segment,
+                          goal: _hints.Segment) -> Relation:
         """
         Returns relation between two segments.
 
@@ -792,40 +896,44 @@ class Context:
 
         >>> context = get_context()
         >>> Point = context.point_cls
-        >>> context.segments_relation(Point(0, 0), Point(2, 2), Point(1, 0),
-        ...                           Point(2, 0)) is Relation.DISJOINT
+        >>> Segment = context.segment_cls
+        >>> (context.segments_relation(Segment(Point(0, 0), Point(2, 2)),
+        ...                            Segment(Point(1, 0), Point(2, 0)))
+        ...  is Relation.DISJOINT)
         True
-        >>> context.segments_relation(Point(0, 0), Point(2, 2), Point(0, 0),
-        ...                           Point(2, 0)) is Relation.TOUCH
+        >>> (context.segments_relation(Segment(Point(0, 0), Point(2, 2)),
+        ...                            Segment(Point(0, 0), Point(2, 0)))
+        ...  is Relation.TOUCH)
         True
-        >>> context.segments_relation(Point(0, 0), Point(2, 2), Point(2, 0),
-        ...                           Point(0, 2)) is Relation.CROSS
+        >>> (context.segments_relation(Segment(Point(0, 0), Point(2, 2)),
+        ...                            Segment(Point(2, 0), Point(0, 2)))
+        ...  is Relation.CROSS)
         True
-        >>> context.segments_relation(Point(0, 0), Point(2, 2), Point(0, 0),
-        ...                           Point(1, 1)) is Relation.COMPOSITE
+        >>> (context.segments_relation(Segment(Point(0, 0), Point(2, 2)),
+        ...                           Segment(Point(0, 0), Point(1, 1)))
+        ...  is Relation.COMPOSITE)
         True
-        >>> context.segments_relation(Point(0, 0), Point(2, 2), Point(0, 0),
-        ...                           Point(2, 2)) is Relation.EQUAL
+        >>> (context.segments_relation(Segment(Point(0, 0), Point(2, 2)),
+        ...                            Segment(Point(0, 0), Point(2, 2)))
+        ...  is Relation.EQUAL)
         True
-        >>> context.segments_relation(Point(0, 0), Point(2, 2), Point(0, 0),
-        ...                           Point(3, 3)) is Relation.COMPONENT
+        >>> (context.segments_relation(Segment(Point(0, 0), Point(2, 2)),
+        ...                            Segment(Point(0, 0), Point(3, 3)))
+        ...  is Relation.COMPONENT)
         True
-        >>> context.segments_relation(Point(0, 0), Point(2, 2), Point(1, 1),
-        ...                           Point(3, 3)) is Relation.OVERLAP
+        >>> (context.segments_relation(Segment(Point(0, 0), Point(2, 2)),
+        ...                            Segment(Point(1, 1), Point(3, 3)))
+        ...  is Relation.OVERLAP)
         True
         """
-        return self._linear.relater(test_start, test_end, goal_start, goal_end,
+        return self._linear.relater(test.start, test.end, goal.start, goal.end,
                                     self.cross_product)
 
     def segments_squared_distance(self,
-                                  first_start: _hints.Point,
-                                  first_end: _hints.Point,
-                                  second_start: _hints.Point,
-                                  second_end: _hints.Point
-                                  ) -> _hints.Coordinate:
+                                  first: _hints.Segment,
+                                  second: _hints.Segment) -> _hints.Scalar:
         """
-        Returns squared Euclidean distance between two segments
-        given their endpoints.
+        Returns squared Euclidean distance between two segments.
 
         Time complexity:
             ``O(1)``
@@ -834,19 +942,32 @@ class Context:
 
         >>> context = get_context()
         >>> Point = context.point_cls
-        >>> context.segments_squared_distance(Point(0, 0), Point(1, 0),
-        ...                                   Point(0, 0), Point(0, 1)) == 0
+        >>> Segment = context.segment_cls
+        >>> context.segments_squared_distance(
+        ...     Segment(Point(0, 0), Point(1, 0)),
+        ...     Segment(Point(0, 0), Point(0, 1))) == 0
         True
-        >>> context.segments_squared_distance(Point(0, 0), Point(1, 0),
-        ...                                   Point(0, 1), Point(1, 1)) == 1
+        >>> context.segments_squared_distance(
+        ...     Segment(Point(0, 0), Point(1, 0)),
+        ...     Segment(Point(0, 1), Point(1, 1))) == 1
         True
-        >>> context.segments_squared_distance(Point(0, 0), Point(1, 0),
-        ...                                   Point(2, 1), Point(2, 2)) == 2
+        >>> context.segments_squared_distance(
+        ...     Segment(Point(0, 0), Point(1, 0)),
+        ...     Segment(Point(2, 1), Point(2, 2))) == 2
         True
         """
         return self._metric.segment_segment_squared_metric(
-                first_start, first_end, second_start, second_end,
-                self.dot_product, self.segments_relation)
+                first.start, first.end, second.start, second.end,
+                self.dot_product, self._segments_intersect)
+
+    def _segments_intersect(self,
+                            first_start: _hints.Point,
+                            first_end: _hints.Point,
+                            second_start: _hints.Point,
+                            second_end: _hints.Point) -> bool:
+        return self._linear.collision_detector(first_start, first_end,
+                                               second_start, second_end,
+                                               self.angle_orientation)
 
 
 _context = _ContextVar('context',
