@@ -1,14 +1,14 @@
 from .enums import (Kind,
                     Orientation)
-from .hints import (Coordinate,
-                    Point,
-                    QuaternaryPointFunction)
+from .hints import (Point,
+                    QuaternaryPointFunction,
+                    Scalar)
 
 
 def kind(vertex: Point,
          first_ray_point: Point,
          second_ray_point: Point,
-         dot_producer: QuaternaryPointFunction[Coordinate]) -> Kind:
+         dot_producer: QuaternaryPointFunction[Scalar]) -> Kind:
     return Kind(to_sign(dot_producer(vertex, first_ray_point, vertex,
                                      second_ray_point)))
 
@@ -16,11 +16,11 @@ def kind(vertex: Point,
 def orientation(vertex: Point,
                 first_ray_point: Point,
                 second_ray_point: Point,
-                cross_producer: QuaternaryPointFunction[Coordinate]
+                cross_producer: QuaternaryPointFunction[Scalar]
                 ) -> Orientation:
     return Orientation(to_sign(cross_producer(vertex, first_ray_point, vertex,
                                               second_ray_point)))
 
 
-def to_sign(value: Coordinate) -> int:
+def to_sign(value: Scalar) -> int:
     return (1 if value > 0 else -1) if value else 0
