@@ -1,3 +1,4 @@
+from fractions import Fraction
 from typing import (Callable,
                     Sequence,
                     Type)
@@ -5,9 +6,12 @@ from typing import (Callable,
 from reprit.base import generate_repr
 
 from ground.core.hints import (Contour,
-                               Coordinate,
+                               Multipoint,
+                               Multipolygon,
+                               Multisegment,
                                Point,
                                Polygon,
+                               Scalar,
                                Segment)
 from .exact import (contour as exact_contour,
                     multipoint as exact_multipoint,
@@ -28,14 +32,14 @@ from .robust import (contour as robust_contour,
                      polygon as robust_polygon,
                      region as robust_region)
 
-ContourCentroid = Callable[[Sequence[Point], Type[Point],
-                            Callable[[Coordinate], Coordinate]], Point]
-MultipointCentroid = RegionCentroid = Callable[[Sequence[Point], Type[Point]],
-                                               Point]
-MultipolygonCentroid = Callable[[Sequence[Polygon], Type[Point]], Point]
-MultisegmentCentroid = Callable[[Sequence[Segment], Type[Point],
-                                 Callable[[Coordinate], Coordinate]], Point]
-PolygonCentroid = Callable[[Contour, Sequence[Contour], Type[Point]], Point]
+ContourCentroid = Callable[[Contour, Type[Point], Callable[[Scalar], Scalar]],
+                           Point]
+MultipointCentroid = Callable[[Multipoint, Type[Point]], Point]
+MultipolygonCentroid = Callable[[Multipolygon, Type[Point]], Point]
+MultisegmentCentroid = Callable[[Multisegment, Type[Point],
+                                 Callable[[Scalar], Scalar]], Point]
+PolygonCentroid = Callable[[Polygon, Type[Point]], Point]
+RegionCentroid = Callable[[Contour, Type[Point]], Point]
 
 
 class Context:
@@ -84,6 +88,13 @@ class Context:
     def region_centroid(self) -> RegionCentroid:
         return self._region_centroid
 
+    @staticmethod
+    def segment_centroid(segment: Segment, point_cls: Type[Point],
+                         *,
+                         _half: Fraction = Fraction(1, 2)) -> Point:
+        return point_cls(_half * (segment.start.x + segment.end.x),
+                         _half * (segment.start.y + segment.end.y))
+
 
 exact_context = Context(contour_centroid=exact_contour.centroid,
                         multipoint_centroid=exact_multipoint.centroid,
