@@ -1,27 +1,27 @@
 from fractions import Fraction
-from typing import (Callable,
-                    Sequence,
+from typing import (Sequence,
                     Tuple,
                     Type)
 
-from ground.core.hints import (Coordinate,
+from ground.core.hints import (Multipolygon,
                                Point,
-                               Polygon)
+                               Polygon,
+                               Scalar)
 from .polygon import centroid_components as polygon_centroid_components
 
 
-def centroid(polygons: Sequence[Polygon],
+def centroid(multipolygon: Multipolygon,
              point_cls: Type[Point],
-             inverse: Callable[[int], Fraction] = Fraction(1).__truediv__
-             ) -> Point:
-    x_numerator, y_numerator, double_area = centroid_components(polygons)
-    inverted_denominator = inverse(3 * double_area)
+             third: Fraction = Fraction(1, 3)) -> Point:
+    x_numerator, y_numerator, double_area = centroid_components(
+            multipolygon.polygons)
+    inverted_denominator = third / double_area
     return point_cls(x_numerator * inverted_denominator,
                      y_numerator * inverted_denominator)
 
 
 def centroid_components(polygons: Sequence[Polygon]
-                        ) -> Tuple[Coordinate, Coordinate, Coordinate]:
+                        ) -> Tuple[Scalar, Scalar, Scalar]:
     iterator = iter(polygons)
     polygon = next(iterator)
     x_numerator, y_numerator, double_area = polygon_centroid_components(
