@@ -10,15 +10,15 @@ from . import hints
 class Point:
     __slots__ = '_x', '_y'
 
-    def __init__(self, x: hints.Coordinate, y: hints.Coordinate) -> None:
+    def __init__(self, x: hints.Scalar, y: hints.Scalar) -> None:
         self._x, self._y = x, y
 
     @property
-    def x(self) -> hints.Coordinate:
+    def x(self) -> hints.Scalar:
         return self._x
 
     @property
-    def y(self) -> hints.Coordinate:
+    def y(self) -> hints.Scalar:
         return self._y
 
     def __eq__(self, other: 'Point') -> bool:
@@ -42,6 +42,18 @@ class Point:
     __repr__ = generate_repr(__init__)
 
 
+class Empty:
+    __slots__ = ()
+
+    def __init__(self) -> None:
+        pass
+
+    def __eq__(self, other: 'Empty'):
+        return isinstance(other, Empty) or NotImplemented
+
+    __repr__ = generate_repr(__init__)
+
+
 class Multipoint:
     __slots__ = '_points',
 
@@ -117,27 +129,27 @@ class Box:
     __slots__ = '_min_x', '_max_x', '_min_y', '_max_y'
 
     def __init__(self,
-                 min_x: hints.Coordinate,
-                 max_x: hints.Coordinate,
-                 min_y: hints.Coordinate,
-                 max_y: hints.Coordinate) -> None:
+                 min_x: hints.Scalar,
+                 max_x: hints.Scalar,
+                 min_y: hints.Scalar,
+                 max_y: hints.Scalar) -> None:
         self._min_x, self._max_x, self._min_y, self._max_y = (min_x, max_x,
                                                               min_y, max_y)
 
     @property
-    def max_x(self) -> hints.Coordinate:
+    def max_x(self) -> hints.Scalar:
         return self._max_x
 
     @property
-    def max_y(self) -> hints.Coordinate:
+    def max_y(self) -> hints.Scalar:
         return self._max_y
 
     @property
-    def min_x(self) -> hints.Coordinate:
+    def min_x(self) -> hints.Scalar:
         return self._min_x
 
     @property
-    def min_y(self) -> hints.Coordinate:
+    def min_y(self) -> hints.Scalar:
         return self._min_y
 
     def __eq__(self, other: 'Box') -> bool:
@@ -187,6 +199,35 @@ class Multipolygon:
     __repr__ = generate_repr(__init__)
 
 
+class Mix:
+    __slots__ = '_discrete', '_linear', '_shaped'
+
+    def __init__(self,
+                 discrete: hints.Maybe[hints.Multipoint],
+                 linear: hints.Maybe[hints.Linear],
+                 shaped: hints.Maybe[hints.Shaped]) -> None:
+        self._discrete, self._linear, self._shaped = discrete, linear, shaped
+
+    @property
+    def discrete(self) -> hints.Maybe[hints.Multipoint]:
+        return self._discrete
+
+    @property
+    def linear(self) -> hints.Maybe[hints.Linear]:
+        return self._linear
+
+    @property
+    def shaped(self) -> hints.Maybe[hints.Shaped]:
+        return self._shaped
+
+    def __eq__(self, other: 'Mix') -> bool:
+        return (self.discrete == other.discrete
+                and self.linear == other.linear
+                and self.shaped == other.shaped)
+
+    __repr__ = generate_repr(__init__)
+
+
 _T = TypeVar('_T')
 
 
