@@ -1,19 +1,29 @@
 from .core.hints import (Box,
                          Contour,
-                         Coordinate,
+                         Empty,
+                         Linear,
+                         Maybe,
+                         Mix,
                          Multipoint,
                          Multipolygon,
                          Multisegment,
                          Point,
                          Polygon,
-                         Segment)
+                         Scalar,
+                         Segment,
+                         Shaped)
 
 Box = Box
 Contour = Contour
-Coordinate = Coordinate
+Empty = Empty
+Linear = Linear
+Maybe = Maybe
+Mix = Mix
 Multipoint = Multipoint
+Multipolygon = Multipolygon
 Multisegment = Multisegment
 Point = Point
-Segment = Segment
 Polygon = Polygon
-Multipolygon = Multipolygon
+Scalar = Scalar
+Segment = Segment
+Shaped = Shaped
